// src/createLazy.tsx
import { createEffect, createSignal } from "solid-js";

// src/utils/listenOnce.ts
function listenOnce(el, type, cb) {
  const listener = () => {
    el.removeEventListener(type, listener);
    cb();
  };
  el.addEventListener(type, listener);
}

// src/utils/listenResize.ts
var listenResize = (cb) => {
  cb();
  window.addEventListener("resize", cb);
  return () => {
    window.removeEventListener("resize", cb);
  };
};

// src/utils/requestNextFrame.tsx
function requestNextFrame(cb) {
  requestAnimationFrame(() => {
    requestAnimationFrame(() => {
      cb();
    });
  });
}

// src/createLazy.tsx
function createLazy(value) {
  const [state, setState] = createSignal(void 0);
  createEffect(() => {
    const v = value();
    requestNextFrame(() => {
      setState(() => v);
    });
  });
  return state;
}

// src/createStyleEffect.ts
import { useActivity } from "@contentstech/stackflow-solid";
import { createEffect as createEffect2, onCleanup, untrack } from "solid-js";
var connections = {};
function createStyleEffect({
  styleName,
  refs,
  effect,
  effectDeps
}) {
  const getActivity = useActivity();
  createEffect2(() => {
    const activity = getActivity();
    if (!(activity == null ? void 0 : activity.id)) {
      return;
    }
    if (!connections[styleName]) {
      connections[styleName] = /* @__PURE__ */ new Map();
    }
    connections[styleName].set(activity.zIndex, {
      refs,
      hasEffect: !!effect
    });
    onCleanup(() => {
      connections[styleName].delete(activity.zIndex);
    });
  });
  createEffect2(() => {
    const activity = getActivity();
    if (!activity) {
      return;
    }
    if (!effect) {
      return;
    }
    const refs2 = (() => {
      let arr = [];
      const zIndex = untrack(() => activity.zIndex);
      for (let i = 1; i <= zIndex; i += 1) {
        const connection = connections[styleName].get(zIndex - i);
        if (connection == null ? void 0 : connection.refs) {
          arr = [...arr, ...connection.refs];
        }
        if (connection == null ? void 0 : connection.hasEffect) {
          break;
        }
      }
      return arr;
    })();
    effect({
      activityTransitionState: activity.transitionState,
      refs: refs2,
      deps: effectDeps == null ? void 0 : effectDeps()
    });
  });
}

// src/createStyleEffectHide.ts
import { onCleanup as onCleanup2 } from "solid-js";
function createStyleEffectHide({
  refs
}) {
  createStyleEffect({
    styleName: "hide",
    refs,
    effect: ({ activityTransitionState, refs: refs2 }) => {
      const cleanup = () => {
        refs2.forEach((ref) => {
          const $el = ref();
          if (!$el) {
            return;
          }
          $el.style.display = "";
        });
      };
      switch (activityTransitionState) {
        case "enter-done": {
          requestNextFrame(() => {
            refs2.forEach((ref) => {
              const $el = ref();
              if (!$el) {
                return;
              }
              $el.style.display = "none";
            });
          });
          onCleanup2(cleanup);
          break;
        }
        default: {
          cleanup();
        }
      }
    }
  });
}

// src/createStyleEffectOffset.ts
import { onCleanup as onCleanup3 } from "solid-js";
function createStyleEffectOffset({
  refs,
  transitionDuration,
  offsetStyles,
  hasEffect = false
}) {
  createStyleEffect({
    styleName: "offset",
    refs,
    effect: hasEffect ? ({ activityTransitionState, refs: refs2 }) => {
      const cleanup = () => {
        requestNextFrame(() => {
          refs2.forEach((ref) => {
            const $el = ref();
            if (!$el) {
              return;
            }
            $el.style.transform = "";
            $el.style.opacity = "";
            listenOnce($el, "transitionend", () => {
              $el.style.transition = "";
            });
          });
        });
      };
      switch (activityTransitionState) {
        case "enter-active":
        case "enter-done": {
          requestNextFrame(() => {
            refs2.forEach((ref) => {
              const $el = ref();
              if (!$el) {
                return;
              }
              $el.style.transition = transitionDuration;
              $el.style.transform = offsetStyles.transform;
              $el.style.opacity = offsetStyles.opacity;
            });
          });
          switch (activityTransitionState) {
            case "enter-done":
              onCleanup3(cleanup);
              return;
            default:
              return;
          }
        }
        case "exit-active":
        case "exit-done": {
          requestNextFrame(() => {
            cleanup();
          });
          break;
        }
        default:
      }
    } : void 0
  });
}

// src/createZIndexBase.ts
import { useActivity as useActivity2 } from "@contentstech/stackflow-solid";
import { createMemo } from "solid-js";
function createZIndexBase() {
  const activity = useActivity2();
  return createMemo(() => {
    var _a, _b;
    return ((_b = (_a = activity()) == null ? void 0 : _a.zIndex) != null ? _b : 0) * 5;
  });
}

// src/useAppBarTitleMaxWidth.ts
import { createEffect as createEffect3, createSignal as createSignal2, onCleanup as onCleanup4 } from "solid-js";
function useAppBarTitleMaxWidth(props) {
  const [maxWidth, setMaxWidth] = createSignal2(void 0);
  createEffect3(() => {
    const $outer = props.outerRef;
    const $inner = props.innerRef;
    if (!props.enable || !$outer || !$inner) return;
    const dispose = listenResize(() => {
      const screenWidth = $outer.clientWidth;
      const leftWidth = $inner.offsetLeft;
      const centerWidth = $inner.clientWidth;
      const rightWidth = screenWidth - leftWidth - centerWidth;
      const sideMargin = Math.max(leftWidth, rightWidth);
      setMaxWidth(screenWidth - 2 * sideMargin);
    });
    onCleanup4(dispose);
  });
  return {
    maxWidth
  };
}

// src/useMounted.ts
import { createSignal as createSignal3, onMount } from "solid-js";
function useMounted() {
  const [mounted, setMounted] = createSignal3(false);
  onMount(() => {
    setMounted(true);
  });
  return mounted;
}

// src/createStyleEffectSwipeBack.ts
import { onCleanup as onCleanup5 } from "solid-js";
function createStyleEffectSwipeBack({
  dimRef,
  edgeRef,
  paperRef,
  offset,
  transitionDuration,
  preventSwipeBack,
  getActivityTransitionState,
  onSwiped
}) {
  createStyleEffect({
    styleName: "swipe-back",
    refs: [paperRef],
    effect: ({ refs }) => {
      if (preventSwipeBack == null ? void 0 : preventSwipeBack()) {
        return;
      }
      if (!dimRef() || !edgeRef() || !paperRef()) {
        return;
      }
      const $dim = dimRef();
      const $edge = edgeRef();
      const $paper = paperRef();
      let x0 = null;
      let t0 = null;
      let x = null;
      let cachedRefs = [];
      const resetState = () => {
        x0 = null;
        t0 = null;
        x = null;
        cachedRefs = [];
      };
      let _rAFLock = false;
      function movePaper(dx) {
        if (!_rAFLock) {
          _rAFLock = true;
          requestAnimationFrame(() => {
            const p = dx / $paper.clientWidth;
            $dim.style.opacity = `${1 - p}`;
            $dim.style.transition = "0s";
            $paper.style.overflowY = "hidden";
            $paper.style.transform = `translate3d(${dx}px, 0, 0)`;
            $paper.style.transition = "0s";
            refs.forEach((ref) => {
              var _a;
              const $el = ref();
              if (!$el) {
                return;
              }
              $el.style.transform = `translate3d(${-1 * (1 - p) * offset}px, 0, 0)`;
              $el.style.transition = "0s";
              if (((_a = $el.parentElement) == null ? void 0 : _a.style.display) === "none") {
                $el.parentElement.style.display = "block";
              }
            });
            _rAFLock = false;
          });
        }
      }
      function resetPaper({ swiped }) {
        return new Promise((resolve) => {
          requestAnimationFrame(() => {
            $dim.style.opacity = `${swiped ? 0 : 1}`;
            $dim.style.transition = transitionDuration;
            $paper.style.overflowY = "hidden";
            $paper.style.transform = `translateX(${swiped ? "100%" : "0"})`;
            $paper.style.transition = transitionDuration;
            refs.forEach((ref) => {
              const $el = ref();
              if (!$el) {
                return;
              }
              $el.style.transition = transitionDuration;
              $el.style.transform = `translate3d(${swiped ? "0" : `-${offset / 16}rem`}, 0, 0)`;
            });
            const _cachedRefs = [...cachedRefs];
            resolve();
            listenOnce($paper, "transitionend", () => {
              const _swiped = swiped || getActivityTransitionState() === "exit-active" || getActivityTransitionState() === "exit-done";
              $dim.style.opacity = "";
              $paper.style.overflowY = "";
              $paper.style.transform = "";
              refs.forEach((ref, i) => {
                const $el = ref();
                if (!$el) {
                  return;
                }
                const _cachedRef = _cachedRefs[i];
                if (_swiped) {
                  $el.style.transition = "";
                  $el.style.transform = "";
                  if ($el.parentElement) {
                    $el.parentElement.style.display = "";
                  }
                } else if (_cachedRef) {
                  $el.style.transition = _cachedRef.style.transition;
                  $el.style.transform = _cachedRef.style.transform;
                  if ($el.parentElement && _cachedRef.parentElement) {
                    $el.parentElement.style.display = _cachedRef.parentElement.style.display;
                  }
                }
              });
            });
          });
        });
      }
      const onTouchStart = (e) => {
        var _a;
        const { activeElement } = document;
        (_a = activeElement == null ? void 0 : activeElement.blur) == null ? void 0 : _a.call(activeElement);
        x0 = x = e.touches[0].clientX;
        t0 = Date.now();
        cachedRefs = refs.map((ref) => {
          const $el = ref();
          if (!$el) {
            return {
              style: {
                transform: "",
                transition: ""
              }
            };
          }
          return {
            style: {
              transform: $el.style.transform,
              transition: $el.style.transition
            },
            parentElement: $el.parentElement ? {
              style: {
                display: $el.parentElement.style.display
              }
            } : void 0
          };
        });
      };
      const onTouchMove = (e) => {
        if (!x0) {
          resetState();
          return;
        }
        x = e.touches[0].clientX;
        movePaper(x - x0);
      };
      const onTouchEnd = () => {
        if (!x0 || !t0 || !x) {
          resetState();
          return;
        }
        const t = Date.now();
        const v = (x - x0) / (t - t0);
        const swiped = v > 1 || x / $paper.clientWidth > 0.4;
        if (swiped) {
          onSwiped == null ? void 0 : onSwiped();
        }
        Promise.resolve().then(() => resetPaper({ swiped })).then(() => resetState());
      };
      $edge.addEventListener("touchstart", onTouchStart, { passive: true });
      $edge.addEventListener("touchmove", onTouchMove, { passive: true });
      $edge.addEventListener("touchend", onTouchEnd, { passive: true });
      $edge.addEventListener("touchcancel", onTouchEnd, { passive: true });
      onCleanup5(() => {
        $edge.removeEventListener("touchstart", onTouchStart);
        $edge.removeEventListener("touchmove", onTouchMove);
        $edge.removeEventListener("touchend", onTouchEnd);
        $edge.removeEventListener("touchcancel", onTouchEnd);
      });
    }
  });
}
export {
  createLazy,
  createStyleEffect,
  createStyleEffectHide,
  createStyleEffectOffset,
  createStyleEffectSwipeBack,
  createZIndexBase,
  useAppBarTitleMaxWidth,
  useMounted
};
//# sourceMappingURL=index.mjs.map
